#define _c_APKA2E
/***********************************************************************/
/*                                                                     */
/* MODULE NAME: APKA2E.C                                               */
/*                                                                     */
/*                                                                     */
/* SYNOPSIS:  ACIF ASCII to EBCDIC Input User Exit                     */
/*                                                                     */
/*                                                                     */
/* DESCRIPTION:  This module converts the input data to EBCDIC.        */
/*                                                                     */
/* COPYRIGHT:                                                      @06A*/
/*  5648-062,5765-140,5639-I27 (C) COPYRIGHT IBM CORPORATION 1987,2000 */
/*  All Rights Reserved                                                */
/*  Licensed Materials - Property of IBM                               */
/*                                                                     */
/*  US Government Users Restricted Rights - Use, duplication or        */
/*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  */
/*                                                                     */
/* NOTE: This program sample is provided on an as-is basis.            */
/*       The licensee of the PSF product is free to copy, revise,      */
/*       modify, and make derivative works of this program sample      */
/*       as they see fit.                                              */
/*                                                                     */
/* STATUS = RELEASE 1  LEVEL 0                                         */
/*                                                                     */
/*                                                                     */
/* EXTERNAL STATIC VARIABLES: None                                     */
/*                                                                     */
/*                                                                     */
/* NOTES:                                                              */
/*                                                                     */
/*                                                                     */
/* CHANGE ACTIVITY:                                                    */
/*                                                                     */
/* PROBNO  REL   DATE/AUTHOR DESCRIPTION                          FICHE*/
/* ------- ----- ----------- -----------                          -----*/
/* PQ04863       T. Medearis Change to iconv invocation to allow       */
/*                           compilation on AIX 4.1 or higher.         */
/*                           Use separate input and output buffers     */
/*                           on iconv invocation to please iconv       */
/*                           people.  P418.                        @01A*/
/* PQ10246       T. Medearis Port to NT                            @02A*/
/* PQ19344       P. Muir     Port to HP/UX                         @03A*/
/* PQ22311       P. Muir     Process files > 2G                    @04A*/
/* PQ25122       P. Muir     Port to SUN/SOLARIS. P479             @05A*/
/*               D. Pirie    Copyright and IPM NT product number   @06A*/
/*               P. Muir     Port ACIF to LINUX  P505              @07A*/
/***********************************************************************/
#if !defined(WIN32)
#define USE_ICONV                                               /*@04A*/
#endif
 
#if defined(USE_ICONV)
#include <iconv.h>                                              /*@02A*/
#else
typedef int iconv_t;
#endif
 
#include <stdio.h>                                              /*@02A*/
#include <stdlib.h>                                             /*@02A*/
#include <string.h>                                             /*@02A*/
#include <errno.h>                                              /*@02A*/
#include "apkexits.h"
 
#if defined(HPUX)
 #define IBM037 "cp037"
 #define IBM850 "cp850"
#elif defined(LINUX)                                            /*@07A*/
 #define IBM037 "CSIBM037"
 #define IBM850 "850"
#else
 #define IBM037 "IBM-037"
 #define IBM850 "IBM-850"
#endif
 
static void ascii_to_ebcdic( char *in_str,
                             char *out_str,
                             size_t str_len
                           );
 
#define ICONV_ERROR (iconv_t) -1
 
long ACIF_EXPORT INPEXIT( INPEXIT_PARMS *exitstruc)
{
   long           rc = 0;
   size_t         bytes;
   size_t         start_index;
   char           *record;
   #if defined(AIX) || defined(SUNOS) || defined(LINUX)   /*    @05C*/
    const char *in;                                       /*    @01A*/
   #else                                                  /*    @01A*/
    char  *in;                                            /*    @01A*/
   #endif                                                 /*    @01A*/
    char *out;                                            /*    @01A*/
#if defined(WIN32)                                              /*@02A*/
    unsigned int inbytes, outbytes;                       /*    @02C*/
#else
    size_t inbytes, outbytes;                             /*    @02C*/
#endif                                                          /*@02A*/
    static iconv_t iconv_fd = ICONV_ERROR;
    static char outbuf[32768];                            /*    @01A*/
    static char inbuf[32768];                             /*    @01A*/
    out = outbuf;                                         /*    @01A*/
    in = inbuf;                                           /*    @01A*/
 
 
   if(exitstruc->eof == 'Y')
   {
 
#if defined(USE_ICONV)
 
      /***************************************************************/
      /* Last record has already been done, so clean up.             */
      /***************************************************************/
 
      if(iconv_fd != ICONV_ERROR)
         iconv_close( iconv_fd );
 
#endif
 
   }
   else
   if(((unsigned char) exitstruc->record[0] == 0x5A) &&        /*@04C*/
      ((unsigned char) exitstruc->record[3] == 0xD3))          /*@04C*/
   {
      /***************************************************************/
      /* I have an AFPDS structured field, ignore it.                */
      /***************************************************************/
   }
   else
   {
      /***************************************************************/
      /* I have line-data.                                           */
      /* I don't want to sniff the data to determine ASCII/EBCDIC/?  */
      /* because an EBCDIC space is 0x40 which is an ASCII '@'.      */
      /* Therefore this exit assumes ASCII.                          */
      /***************************************************************/
      start_index = 0;
      if(*(exitstruc->pfattr->cctype) != 'A')
      {
         /************************************************************/
         /* I only want to convert the CC and TRC if it is specified */
         /* in EBCDIC.  Therefore if it is not EBCDIC jump over the  */
         /* CC and/or TRC.                                           */
         /************************************************************/
         if(*(exitstruc->pfattr->cc) == 'Y')
            start_index++;
         if(*(exitstruc->pfattr->trc) == 'Y')
            start_index++;
      }
 
#if defined(USE_ICONV)
 
      /***************************************************************/
      /* I only want to open the converter once.                     */
      /***************************************************************/
      if(iconv_fd == ICONV_ERROR)
      {
         iconv_fd = iconv_open( IBM037, IBM850 );
         if(iconv_fd == ICONV_ERROR)
         {
            rc = errno;
            rc = 0; /* Default to hard-coded table */
         }
      }
 
#else
 
      rc = 0;
 
#endif
 
      if(rc == 0)
      {
         /************************************************************/
         /* Convert the record from ASCII to EBCDIC.                 */
         /************************************************************/
         if(exitstruc->recordln > start_index)
         {
            bytes = exitstruc->recordln - start_index;
            record = exitstruc->record + start_index;
 
            inbytes = bytes;          /*                           @01A*/
            outbytes = bytes;         /*                           @01A*/
            memcpy(inbuf,             /* move record to temp input @04C*/
                   record,
                   bytes);
            if(iconv_fd == ICONV_ERROR)
            {
               rc = 0;
               ascii_to_ebcdic( (char *) in, (char *) out, bytes );
            }
            else
            {
 
#if defined(USE_ICONV)
 
              rc = iconv( iconv_fd,   /* convert to temp output area   */
                     &in,             /*                           @01C*/
                     &inbytes,
                     &out,
                     &outbytes
                   );
#else
 
               rc = -1;                                          /*@04A*/
 
#endif
            }
 
            if (rc >= 0)              /* If iconv was successful   @01A*/
             {
               rc = 0;
               memcpy(record,         /* move converted record back to */
                      outbuf,         /* record area               @04C*/
                      bytes);
             }
            else                  /* else iconv was not successful @01A*/
             {
               rc = errno;            /* save error code           @01A*/
             }
         }
      }
   }
 
   exitstruc->request = INP_USE;
   return( rc );
}
 
/**********************************************************************/
/*                                                                    */
/* FUNCTION NAME:  ascii_to_ebcdic                                    */
/*                                                                    */
/* SYNOPSIS:  Convert ASCII string to EBCDIC                          */
/*                                                                    */
/* DESCRIPTION:  Translate and ASCII buffer to an EBCDIC buffer.      */
/*                                                                    */
/* NOTES:  This came from the IBM-850_IBM-037_src iconv file on AIX   */
/*                                                                    */
/**********************************************************************/
static
void
ascii_to_ebcdic( char *in_str, char *out_str, size_t str_len )
{
   unsigned char        *in_ptr;
   unsigned char        *out_ptr;
   static unsigned char ascii850_to_ebcdic037[] =
   {
     0x00,0x01,0x02,0x03,0x37,0x2d,0x2e,0x2f,
     0x16,0x05,0x25,0x0b,0x0c,0x0d,0x0e,0x0f,
     0x10,0x11,0x12,0x13,0x3c,0x3d,0x32,0x26,
     0x18,0x19,0x1c,0x27,0x07,0x1d,0x1e,0x1f,
     0x40,0x5a,0x7f,0x7b,0x5b,0x6c,0x50,0x7d,
     0x4d,0x5d,0x5c,0x4e,0x6b,0x60,0x4b,0x61,
     0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,
     0xf8,0xf9,0x7a,0x5e,0x4c,0x7e,0x6e,0x6f,
     0x7c,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7,
     0xc8,0xc9,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,
     0xd7,0xd8,0xd9,0xe2,0xe3,0xe4,0xe5,0xe6,
     0xe7,0xe8,0xe9,0xba,0xe0,0xbb,0xb0,0x6d,
     0x79,0x81,0x82,0x83,0x84,0x85,0x86,0x87,
     0x88,0x89,0x91,0x92,0x93,0x94,0x95,0x96,
     0x97,0x98,0x99,0xa2,0xa3,0xa4,0xa5,0xa6,
     0xa7,0xa8,0xa9,0xc0,0x4f,0xd0,0xa1,0x3f,
     0x68,0xdc,0x51,0x42,0x43,0x44,0x47,0x48,
     0x52,0x53,0x54,0x57,0x56,0x58,0x63,0x67,
     0x71,0x9c,0x9e,0xcb,0xcc,0xcd,0xdb,0xdd,
     0xdf,0xec,0xfc,0x70,0xb1,0x80,0xbf,0x3f,
     0x45,0x55,0xce,0xde,0x49,0x69,0x9a,0x9b,
     0xab,0xaf,0x5f,0xb8,0xb7,0xaa,0x8a,0x8b,
     0x3f,0x3f,0x3f,0x3f,0x3f,0x65,0x62,0x64,
     0xb4,0x3f,0x3f,0x3f,0x3f,0x4a,0xb2,0x3f,
     0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x46,0x66,
     0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x9f,
     0x8c,0xac,0x72,0x73,0x74,0x3f,0x75,0x76,
     0x77,0x3f,0x3f,0x3f,0x3f,0x6a,0x78,0x3f,
     0xee,0x59,0xeb,0xed,0xcf,0xef,0xa0,0x8e,
     0xae,0xfe,0xfb,0xfd,0x8d,0xad,0xbc,0xbe,
     0xca,0x8f,0x3f,0xb9,0xb6,0xb5,0xe1,0x9d,
     0x90,0xbd,0xb3,0xda,0xfa,0xea,0x3f,0x41
   };
 
   /*******************************************************************/
   /* Loop convert source to destination                              */
   /*******************************************************************/
   for( in_ptr = (unsigned char *) in_str,
        out_ptr = (unsigned char *) out_str;
        str_len--;
      )
   {
      out_ptr[str_len] = ascii850_to_ebcdic037[in_ptr[str_len]];
   }
}
