#if !defined(_h_APKEXITS)
#define _h_APKEXITS
/***********************************************************************/
/*                                                                     */
/*  APKEXITS.H :  User Exit Declarations                               */
/*                                                                     */
/*  This file contains function and structure declarations for         */
/*  the ACIF user exits.                                               */
/*                                                                     */
/*                                                                     */
/* COPYRIGHT:                                                      $04A*/
/*  5648-062,5765-140,5639-I27 (C) COPYRIGHT IBM CORPORATION 1987,2000 */
/*  All Rights Reserved                                                */
/*  Licensed Materials - Property of IBM                               */
/*                                                                     */
/*  US Government Users Restricted Rights - Use, duplication or        */
/*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  */
/*                                                                     */
/* NOTE: This program sample is provided on an as-is basis.            */
/*       The licensee of the PSF product is free to copy, revise,      */
/*       modify, and make derivative works of this program sample      */
/*       as they see fit.                                              */
/*                                                                     */
/*                                                                     */
/* STATUS = RELEASE 1  LEVEL 0                                         */
/*                                                                     */
/*                                                                     */
/* NOTES:   NO strings are NULL (0x00) terminated.                     */
/*                                                                     */
/*                                                                     */
/* CHANGE ACTIVITY:                                                    */
/*                                                                     */
/* PROBNO  DATE    AUTHOR    DESCRIPTION                          FICHE*/
/* ------- ----- ----------- -----------                          -----*/
/* PQ19344 980914  PJM        Port to HP/UX  P452                  $01A*/
/* PQ22311 981209  PJM        Process files > 2G  P464             $02A*/
/* PQ23126 990129  PJM        TRC not matching    P476             $03A*/
/*         000106  DAP        Copyright and IPM NT product number  $04A*/
/***********************************************************************/
 
#if defined(WIN32)
 #define ACIF_EXPORT __declspec( dllexport )                   /*@02C*/
#else
 #define ACIF_EXPORT
#endif
 
#define ACIF_USE     0x00
#define ACIF_DELETE  0x01
#define ACIF_IGNORE  0x01
#define ACIF_INSERT  0x02
#define ACIF_NOT_EOF 'N'
#define ACIF_EOF     'Y'
 
/*********************************************************************/
/* Input Exit Flags.                                                 */
/*********************************************************************/
#define INP_USE    ACIF_USE
#define INP_DELETE ACIF_DELETE
#define INP_INSERT ACIF_INSERT
#define INP_NOTEOF ACIF_NOT_EOF
#define INP_EOFLAG ACIF_EOF
 
/*********************************************************************/
/* Index Exit Flags.                                                 */
/*********************************************************************/
#define IDX_USE    ACIF_USE
#define IDX_DELETE ACIF_DELETE
#define IDX_NOTEOF ACIF_NOT_EOF
#define IDX_EOFLAG ACIF_EOF
 
/*********************************************************************/
/* Output Exit Flags.                                                */
/*********************************************************************/
#define OUT_USE    ACIF_USE
#define OUT_DELETE ACIF_DELETE
#define OUT_NOTEOF ACIF_NOT_EOF
#define OUT_EOFLAG ACIF_EOF
 
/*********************************************************************/
/* Resource Exit Flags.                                              */
/*********************************************************************/
#define RSP_USE    ACIF_USE
#define RSP_IGNORE ACIF_IGNORE
#define RSP_NOTEOF ACIF_NOT_EOF
#define RSP_EOFLAG ACIF_EOF
 
/*********************************************************************/
/* PFATTR STRUCTURE                                                  */
/*********************************************************************/
typedef struct _PFATTR /* Print File Attributes                              */
{
                    /* No strings are NULL-terminated.    */
   char cc[3];      /* Carriage controls? - "YES" or "NO "                   */
   char cctype[1];  /* Carriage control type - A(ANSI), M(Machine), Z(ASCII) */
                                                                       /*@03C*/
   char chars[20];  /* CHARS values, including commas (eg. GT12,GT15)        */
   char formdef[8]; /* Form Definition (FORMDEF)                             */
   char pagedef[8]; /* Page Definition (PAGEDEF)                             */
   char prmode[8];  /* Processing mode                                       */
   char trc[3];     /* Table Reference Characters - "YES" or "NO "           */
} PFATTR;
 
/*********************************************************************/
/* INPEXIT STRUCTURE                                                 */
/*********************************************************************/
typedef struct _INPEXIT_PARMS /* Parameters for the input record exit        */
{
   char           *work;      /* Address of 16-byte static work area         */
   PFATTR         *pfattr;    /* Address of print file attribute information */
   char           *record;    /* Address of the input record                 */
   void           *reserved1; /* Reserved for future use                     */
   unsigned short recordln;   /* Length of the input record                  */
   unsigned short reserved2;  /* Reserved for future use                     */
   char           request;    /* Add, delete, or process the record          */
   char           eof;        /* EOF indicator                               */
} INPEXIT_PARMS;
 
/*********************************************************************/
/* INDXEXIT STRUCTURE                                                */
/*********************************************************************/
typedef struct _INDXEXIT_PARMS /* Parameters for the index record exit       */
{
   char           *work;      /* Address of 16-byte static work area         */
   PFATTR         *pfattr;    /* Address of print file attribute information */
   char           *record;    /* Address of the record to be written         */
   unsigned short recordln;   /* Length of the output record                 */
   char           request;    /* Delete or process the record                */
   char           eof;        /* Last call indicator to ACIF                 */
} INDXEXIT_PARMS;
 
/*********************************************************************/
/* OUTEXIT STRUCTURE                                                 */
/*********************************************************************/
typedef struct _OUTEXIT_PARMS /* Parameters for the output record exit       */
{
   char           *work;      /* Address of 16-byte static work area         */
   PFATTR         *pfattr;    /* Address of print file attribute information */
   char           *record;    /* Address of the record to be written         */
   unsigned short recordln;   /* Length of the output record                 */
   char           request;    /* Delete or process the record                */
   char           eof;        /* Last call indicator                         */
} OUTEXIT_PARMS;
 
/*********************************************************************/
/* RESEXIT STRUCTURE                                                 */
/*********************************************************************/
typedef struct _RESEXIT_PARMS /* Parameters for the resource record exit     */
{
   char           *work;      /* Address of 16-byte static work area         */
   PFATTR         *pfattr;    /* Address of print file attribute information */
   char           resname[8]; /* Name of requested resource                  */
   char           restype;    /* Type of resource                            */
   char           request;    /* Ignore or process the resource              */
   char           eof;        /* Last call indicator                         */
} RESEXIT_PARMS;
 
/*********************************************************************/
/* FUNCTION DECLARATIONS                                             */
/*********************************************************************/
long ACIF_EXPORT INPEXIT( INPEXIT_PARMS * );
long ACIF_EXPORT INDXEXIT( INDXEXIT_PARMS * );
long ACIF_EXPORT OUTEXIT( OUTEXIT_PARMS * );
long ACIF_EXPORT RESEXIT( RESEXIT_PARMS * );
 
#endif
