/**********************************************************************/
/*                                                                    */
/*  ARSCSXIT.H :    OnDemand User Exit Declarations                   */
/*                                                                    */
/*  This file contains function and structure declarations for        */
/*  the OnDemand user exits.                                          */
/*                                                                    */
/* COPYRIGHT:                                                         */
/*  5622-662 (C) COPYRIGHT IBM CORPORATION 1996,1997                  */
/*  All Rights Reserved                                               */
/*  Licensed Materials - Property of IBM                              */
/*                                                                    */
/*  US Government Users Restricted Rights - Use, duplication or       */
/*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp. */
/*                                                                    */
/* NOTE: This program sample is provided on an as-is basis.           */
/*       The licensee of the OnDemand product is free to copy, revise,*/
/*       modify, and make derivative works of this program sample     */
/*       as they see fit.                                             */
/*                                                                    */
/**********************************************************************/
#ifndef _ARSCSXIT_H
#define _ARSCSXIT_H

#ifdef __cplusplus
extern "C" {
#endif

#if defined(WIN32)
 #include <windows.h>
 #define ARSCSXIT_API
 #define ARSCSXIT_APIFUNC ARSCSXIT_API *
 #define ARSCSXIT_EXPORT  __declspec( dllexport )
#else
 #define ARSCSXIT_API
 #define ARSCSXIT_APIFUNC * ARSCSXIT_API
 #define ARSCSXIT_EXPORT
#endif

#if defined(AIX) || defined(HPUX) || defined(SUNOS) || defined(LINUX) \
                 || defined(__OS400__) || defined(OS390)

typedef long long ArcCSXitBigInt;

#elif defined(WIN32)

typedef __int64 ArcCSXitBigInt;

#else

typedef long ArcCSXitBigInt;

#endif

/*********************************************************************/
/* FAX OPTIONS STRUCTURES                                            */
/*********************************************************************/
#define ARSCSXIT_FAX_RECIPIENT_ATTN_MAX    50
#define ARSCSXIT_FAX_RECIPIENT_COMPANY_MAX 50
#define ARSCSXIT_FAX_RECIPIENT_FAX_MAX     32
#define ARSCSXIT_FAX_SENDER_FROM_MAX       50
#define ARSCSXIT_FAX_SENDER_COMPANY_MAX    50
#define ARSCSXIT_FAX_SENDER_PHONE_MAX      32
#define ARSCSXIT_FAX_SENDER_FAX_MAX        32
#define ARSCSXIT_FAX_SENDER_COVERPAGE_MAX  8

typedef struct _ArsCSXitFaxOptions
{
   char recipient_attn[ARSCSXIT_FAX_RECIPIENT_ATTN_MAX + 1];
   char recipient_company[ARSCSXIT_FAX_RECIPIENT_COMPANY_MAX + 1];
   char recipient_fax[ARSCSXIT_FAX_RECIPIENT_FAX_MAX + 1];
   char sender_name[ARSCSXIT_FAX_SENDER_FROM_MAX + 1];
   char sender_company[ARSCSXIT_FAX_SENDER_COMPANY_MAX + 1];
   char sender_phone[ARSCSXIT_FAX_SENDER_PHONE_MAX + 1];
   char sender_fax[ARSCSXIT_FAX_SENDER_FAX_MAX + 1];
   char sender_coverpage[ARSCSXIT_FAX_SENDER_COVERPAGE_MAX + 1];
} ArsCSXitFaxOptions;

/*********************************************************************/
/* SECURITY STRUCTURES                                               */
/*********************************************************************/
typedef enum _ArcCSXitSecurityAction
{
   ARCCSXIT_SECURITY_USER_LOGIN,
   ARCCSXIT_SECURITY_USER_ADD,
   ARCCSXIT_SECURITY_USER_DELETE,
   ARCCSXIT_SECURITY_USER_UPDATE
} ArcCSXitSecurityAction;

typedef enum _ArcCSXitSecurityRC
{
   ARCCSXIT_SECURITY_RC_OKAY,
   ARCCSXIT_SECURITY_RC_PERMS,
   ARCCSXIT_SECURITY_RC_PASSWD_CHNG,
   ARCCSXIT_SECURITY_RC_FAILED
} ArcCSXitSecurityRC;

/*********************************************************************/
/* COMMON STRUCTURES                                                 */
/*********************************************************************/
#define ARCCSXIT_MAX_SRVR_MESSAGE_SIZE 1024

typedef struct _ArcCSXitApplGroup
{
   char *name;
   long agid;
   char *agid_name;
} ArcCSXitApplGroup;

typedef unsigned char ArcCSXitFieldType;
#define ARCCSXIT_FIELD_TYPE_BIGINT   (ArcCSXitFieldType) 'B'
#define ARCCSXIT_FIELD_TYPE_DECIMAL  (ArcCSXitFieldType) 'D'
#define ARCCSXIT_FIELD_TYPE_INTEGER  (ArcCSXitFieldType) 'I'
#define ARCCSXIT_FIELD_TYPE_SMALLINT (ArcCSXitFieldType) 'N'
#define ARCCSXIT_FIELD_TYPE_STRING   (ArcCSXitFieldType) 'S'

typedef struct _ArcCSXitField
{
   char              *db_name;
   ArcCSXitFieldType type;
   union
   {
      double         d;
      ArcCSXitBigInt b;
      long           i;
      short          n;
      char           *str;
   } u;
} ArcCSXitField;

typedef struct _ArcCSXitDocFields
{
   int           flds_num;
   ArcCSXitField *flds;
} ArcCSXitDocFields;

#define ARCCSXIT_DOCNAME_SIZE 11

typedef struct _ArcCSXitDocHandle
{
   char          name[ARCCSXIT_DOCNAME_SIZE + 1];
   unsigned long doc_off;
   unsigned long doc_len;
   unsigned long comp_off;
   unsigned long comp_len;
} ArcCSXitDocHandle;

typedef struct _ArcCSXitDoc
{
   ArcCSXitDocFields doc_flds;
   ArcCSXitDocHandle doc_hndl;
} ArcCSXitDoc;

/*********************************************************************/
/* FUNCTION DECLARATIONS                                             */
/*********************************************************************/

/**********************************************************************/
/* FAXOPTS - Fax Options Exit                                         */
/*                                                                    */
/* INPUT: appl_grp                                                    */
/*        doc_flds                                                    */
/*                                                                    */
/* OUTPUT:                                                            */
/*        exitdata                                                    */
/*                                                                    */
/* RETURN_CODE:                                                       */
/*        0         -> Successful                                     */
/*        Otherwise -> Failed                                         */
/*                                                                    */
/**********************************************************************/
int
ARSCSXIT_EXPORT
ARSCSXIT_API
FAXOPTS( ArcCSXitApplGroup *appl_grp,
         ArcCSXitDocFields *doc_flds,
         ArsCSXitFaxOptions *exitdata
       );

/**********************************************************************/
/* SECURITY - Security Exit                                           */
/*                                                                    */
/* 1) User Login                                                      */
/*      On Input:  action == ARCCSXIT_SECURITY_USER_LOGIN             */
/*                                                                    */
/*    INPUT: cur_userid - Userid                                      */
/*           cur_passwd - Password                                    */
/*                                                                    */
/*    OUTPUT:                                                         */
/*                                                                    */
/* 2) User Add                                                        */
/*      On Input:  action == ARCCSXIT_SECURITY_USER_ADD               */
/*                                                                    */
/*    INPUT: act_userid - Actual User doing the add                   */
/*           new_userid - Userid to add                               */
/*           new_passwd - Password                                    */
/*                                                                    */
/*    OUTPUT:                                                         */
/*                                                                    */
/* 3) User Delete                                                     */
/*      On Input:  action == ARCCSXIT_SECURITY_USER_DELETE            */
/*                                                                    */
/*    INPUT: act_userid - Actual User doing the delete                */
/*           cur_userid - Userid to delete                            */
/*                                                                    */
/*    OUTPUT:                                                         */
/*                                                                    */
/* 4) User Update                                                     */
/*      On Input:  action == ARCCSXIT_SECURITY_USER_UPDATE            */
/*                                                                    */
/*    INPUT: act_userid - Actual User doing the update                */
/*           cur_userid - Current userid                              */
/*           cur_passwd - Current password                            */
/*           new_userid - New userid                                  */
/*           new_passwd - New Password                                */
/*                        If NULL, then no password change            */
/*                                                                    */
/*    OUTPUT:                                                         */
/*                                                                    */
/* RETURN_CODE:                                                       */
/*        ARCCSXIT_SECURITY_RC_OKAY        -> Successful              */
/*        ARCCSXIT_SECURITY_RC_PERMS       -> No permission.          */
/*        ARCCSXIT_SECURITY_RC_PASSWD_CHNG -> Only valid on Login.    */
/*                                            Login is successful,    */
/*                                            password must be        */
/*                                            changed.                */
/*        ARCCSXIT_SECURITY_RC_FAILED      -> Failed                  */
/*                                                                    */
/* NOTES:  (output) msg is ARCCSXIT_MAX_SRVR_MESSAGE_SIZE bytes in    */
/*         size.  If the return code != ARCCSXIT_SECURITY_RC_OKAY     */
/*         then if msg[0] != '\0', then this will be the message      */
/*         displayed by the client.  Otherwise the client will use    */
/*         its native message text.                                   */
/*                                                                    */
/**********************************************************************/
ArcCSXitSecurityRC
ARSCSXIT_EXPORT
ARSCSXIT_API
SECURITY( char *act_userid,
          char *cur_userid,
          char *cur_passwd,
          char *new_userid,
          char *new_passwd,
          ArcCSXitSecurityAction action,
          char *msg
        );

/**********************************************************************/
/* SMEXTCAC - Storage Management External Cache Exit                  */
/*                                                                    */
/* 1) Don't return data, only validate whether the document exists.   */
/*      On Input:  buf == NULL                                        */
/*                                                                    */
/*    INPUT: appl_grp                                                 */
/*           doc                                                      */
/*           buf                                                      */
/*                                                                    */
/*    OUTPUT:                                                         */
/*           *buf_len = 0 -> Data is not in external cache            */
/*                        -> Otherwise data is in external cache      */
/*                                                                    */
/* 2) Return document data.                                           */
/*      On Input:  buf != NULL                                        */
/*                                                                    */
/*    INPUT: appl_grp                                                 */
/*           doc                                                      */
/*           *buf_len -> #of bytes to retrieve                        */
/*                                                                    */
/*    OUTPUT:                                                         */
/*           *buf     -> Allocated memory and document data.          */
/*                       Memory is freed by OnDemand using the        */
/*                       free() function                              */
/*           *buf_len -> Length of *buf (should be same as on input)  */
/*                    -> If 0, then data does not exist within        */
/*                       external cache.                              */
/*                                                                    */
/* RETURN_CODE:                                                       */
/*        0         -> Successful                                     */
/*        Otherwise -> Failed                                         */
/*                                                                    */
/**********************************************************************/
int
ARSCSXIT_EXPORT
ARSCSXIT_API
SMEXTCAC( ArcCSXitApplGroup *appl_grp,
          ArcCSXitDoc *doc,
          char **buf,
          unsigned long *buf_len
        );

#ifdef __cplusplus
}
#endif

#endif
